@@ -58,13 +58,10 @@
 
 #define DEF_DB_HOME DATA_STATE_DIR"/"PACKAGE
 
-#define DB_STAT_NAME "stats.db"
 #define DB_FILE_NAME "triplets.db"
 #define SEP '\000'
 
-/* Counter for triplets that are expired and never passed anything
- * through */
-#define STAT_BLOCKED "blocked"
+#define prefixp(s,p) (!strncmp((s),(p),sizeof(p)-1))
 
 struct triplet_data {
     time_t create_time;
@@ -77,7 +74,6 @@ static const char *db_home = DEF_DB_HOME;
 
 static DB_ENV *dbenv = 0;
 static DB *db = 0;
-static DB *statsdb = 0;
 
 static DBT dbkey = { 0 };
 static DBT dbdata = { 0 };
@@ -136,6 +132,21 @@ db_errcall_fcn(const DB_ENV *dbenv, const char *errpfx, const char *msg)
     syslog(LOG_ERR, "%s: %s", errpfx ? errpfx : "Berkeley DB", msg);
 }
 
+static int
+db_open(DB *db, const char *file, const char *database,
+	DBTYPE type, u_int32_t flags, int mode)
+{
+    /* http://www.oracle.com/technology/documentation/berkeley-db/db/ref/upgrade.4.1/fop.html
+       To upgrade, applications must add a DB_TXN parameter in the
+       appropriate location for the DB->open method calls [...] the
+       second argument */
+#if DB_VERSION_MAJOR > 4 || DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR >= 1
+    return db->open(db, NULL, file, database, type, flags, mode);
+#else
+    return db->open(db, file, database, type, flags, mode);
+#endif
+}
+
 static int
 prepare_env()
 {
@@ -161,23 +172,9 @@ prepare_db()
     if (rc)
 	log_db_error("db_create", rc);
     else {
-	rc = db->open(db, NULL, DB_FILE_NAME, NULL, DB_BTREE,
-		      DB_CREATE, 0644);
+	rc = db_open(db, DB_FILE_NAME, NULL, DB_BTREE, DB_CREATE, 0644);
 	if (rc)
 	    log_db_error("db->open", rc);
-	else {
-	    rc = db_create(&statsdb, dbenv, 0);
-	    if (rc)
-		syslog(LOG_ERR, "BDB-%d: db_create statsdb: %s",
-		       rc, db_strerror(rc));
-	    else {
-		rc = db->open(statsdb, NULL, DB_STAT_NAME, NULL, DB_BTREE,
-			      DB_CREATE, 0644);
-		if (rc)
-		    syslog(LOG_ERR, "BDB-%d: db->open %s: %s",
-			   rc, DB_STAT_NAME, db_strerror(rc));
-	    }
-	}
     }
     return rc;
 }
@@ -234,12 +231,6 @@ cleanup()
 	if (rc)
 	    log_db_error("DB close", rc);
     }
-    if (statsdb) {
-	rc = statsdb->close(statsdb, 0);
-	statsdb = 0;
-	if (rc)
-	    log_db_error("StatsDB close", rc);
-    }
     if (dbenv) {
 	rc = dbenv->close(dbenv, 0);
 	dbenv = 0;
@@ -253,7 +244,7 @@ cleanup()
 	    muffle_error++;
 	    rc = dbenv->remove(dbenv, db_home, 0);
 	    if (rc)
-		syslog(LOG_DEBUG, "DEBUG: BDB-%d: db_env->remove failed: %s",
+		syslog(LOG_DEBUG, "DEBUG: BDB-%d: db_env->remove returned: %s",
 		       rc, db_strerror(rc));
 	    muffle_error--;
 	}
